!==============================================================================!
 subroutine eig_mat_ar (je, e_kin)
!==============================================================================!
! This subroutine stores the G_o T kernel matrix and computes a subset of the
! the eigenvalues with largest magnitude using Arnoldi iteration methods from 
! ARPACK
!
    use dim_mod,      only: n_gaunt, nl_m
    use coor_mod,     only: natyp, ncorr, n_prot, sym_at
    use outfiles_mod, only: outfile2
    use outunits_mod, only: iuo1, iuo2
    use trans_mod,    only: lmax, vk, tl
!
    use arnoldi_mod,  only: arnoldi_iteration
!
    implicit none
!
    integer, parameter :: sp = kind(1.0)
    integer, parameter :: dp = kind(1.0d0)
    integer, parameter :: cp = kind((0.0,0.0))
    integer, parameter :: zp = kind((0.0d0,0.0d0))
!
! Subroutine arguments
!
    integer,  intent(in) :: je
    real(sp), intent(in) :: e_kin
!
! Local data
!
    integer,     parameter :: ibess = 3
    integer,     parameter :: nprint = 10
    real(dp),    parameter :: pi = acos(-1.0_dp)
    real(dp),    parameter :: small = 0.0001_dp
    complex(zp), parameter :: ic = complex(0.0d0,1.0d0)
    complex(zp), parameter :: zeroc = complex(0.0d0,0.0d0)
    complex(zp), parameter :: four_pi_i = 4.0_dp*pi*ic
!
    integer     :: jatl, jlin, jtyp, jnum, lj, lmj, mj, nbtypj
    integer     :: katl, klin, ktyp, knum, lk, lmk, mk, nbtypk
    integer     :: j, jp, l, lin, l_max, l_min, m
    integer     :: ndim, n_dot, n_eig, nev, nfin, nltwo, npr, n_xmax
    real(sp)    :: eig, xj, yj, zj, xmax_l
    real(dp)    :: attkj, xkj, ykj, zkj, rkj, zdkj, krkj
    complex(zp) :: expkj, sum_l, tlk
!
    integer, save :: iout2, iout3
!
    real(sp),     allocatable :: w1(:), w2(:)
    real(dp),     allocatable :: gnt(:)
    complex(zp),  allocatable :: hl1(:), sm(:,:), ylm(:,:), w(:)
    character(:), allocatable :: outfile, path
!
!
    if (je == 1) then
!
! Name of second output file where eigenvalues will be written
!
        n_dot = index(outfile2, '.')
        outfile = outfile2(1:n_dot)//'egv'
        path = outfile2(1:n_dot)//'pth'
        open(newunit=iout2, file=outfile, status='unknown')
        open(newunit=iout3, file=path, status='unknown')
!
    end if
!
! Construction of the multiple scattering kernel matrix G_o T.
! Elements are stored using a linear index LINJ representing (J,LJ)
!
! First compute Go T array dimension
!
    jlin = 0
    do jtyp = 1, n_prot
        nbtypj = natyp(jtyp)
        lmj = lmax(jtyp,je)
        do jnum = 1, nbtypj
            do lj = 0, lmj
                do mj = -lj, lj
                    jlin = jlin + 1
                end do
            end do
        end do
    end do
!
    ndim = jlin
    write(6,*) "GoT matrix sm has dimension ", ndim
!
    allocate(sm(ndim,ndim))
    sm = zeroc
!
    nltwo = 2*nl_m
    allocate(ylm(0:nltwo, -nltwo:nltwo))
    ylm = zeroc
!
    allocate(hl1(0:nltwo))
    hl1 = zeroc
!
    allocate(gnt(0:n_gaunt))
    gnt = 0.0_dp
!
    jlin = 0
    do jtyp = 1, n_prot
        nbtypj = natyp(jtyp)
        lmj = lmax(jtyp,je)
        do jnum = 1, nbtypj
            jatl = ncorr(jnum,jtyp)
            xj = sym_at(1,jatl)
            yj = sym_at(2,jatl)
            zj = sym_at(3,jatl)
            do lj = 0, lmj
                do mj = -lj, lj
                    jlin = jlin + 1
!
                    klin=0
                    do ktyp = 1, n_prot
                        nbtypk = natyp(ktyp)
                        lmk = lmax(ktyp,je)
                        do knum = 1, nbtypk
                            katl = ncorr(knum,ktyp)
!
                            if (katl /= jatl) then
                                xkj = real(sym_at(1,katl) - xj, dp)
                                ykj = real(sym_at(2,katl) - yj, dp)
                                zkj = real(sym_at(3,katl) - zj, dp)
                                rkj = sqrt(xkj*xkj + ykj*ykj + zkj*zkj)
                                krkj = real(vk(je), dp)*rkj
                                attkj = exp(-aimag(cmplx(vk(je)))*rkj)
                                expkj = (xkj + ic*ykj)/rkj
                                zdkj = zkj/rkj
                                call sph_har2(2*nl_m, zdkj, expkj, ylm, lmj+lmk)
                                call besphe2(lmj+lmk+1, ibess, krkj, hl1)
                            end if
!
                            do lk = 0,lmk
                                l_min = abs(lk-lj)
                                l_max = lk + lj
                                tlk = cmplx(tl(lk,1,ktyp,je))
                                do mk = -lk, lk
                                    klin = klin + 1
! 
                                    sm(klin,jlin) = zeroc
                                    sum_l = zeroc
                                    if (katl /= jatl) then
                                        call gaunt2(lk, mk, lj, mj, gnt)
                                        do l = l_min, l_max, 2
                                            m = mj - mk
                                            if (abs(m) <= l) then
                                                sum_l = sum_l + (ic**l)*hl1(l)*ylm(l,m)*gnt(l)
                                            end if
                                        end do
                                        sum_l = sum_l*attkj*four_pi_i
                                    else
                                        sum_l = zeroc
                                    end if
!
                                    sm(klin,jlin) = tlk*sum_l
!
                                end do
                            end do
                        end do
                    end do
                end do
            end do
        end do
    end do
!
    deallocate(ylm, hl1, gnt)
!
! Compute subset of eigenvalues of SM using ARPACK
! 
! NEV is the number of eigenvalues required, set to 2% of NDIM
!
    nev = ceiling(0.02*ndim)
    allocate(w(nev))
!
    call arnoldi_iteration(ndim, sm, nev, w)
!
    deallocate(sm)
!
! Save results to filestream for easy access from python
!
    call save_eigenvalues(w, nev, e_kin)
!
! Write results to OUTFILE on unit IOUT2
!
    write(iout2,75)
    write(iout2,110)
    write(iout2,80) e_kin
    write(iout2,110)
    write(iout2,75)
    write(iout2,105)
    write(iout2,75)
!
    allocate(w1(nev), w2(nev))
!
    n_eig = 0
    xmax_l = 0.0
    n_xmax = 0
    do lin = 1, nev
        eig = real(abs(w(lin)))
        write(iout2,100) real(w(lin)), aimag(w(lin)), eig
        if ((eig-xmax_l) > 0.0001) n_xmax = lin
        xmax_l = max(xmax_l, eig)
        w1(lin) = eig
        if (eig > 1.000) then
            n_eig = n_eig + 1
        end if
    end do
!
    write(iout2,75)
    write(iout2,85) xmax_l
    write(iout2,90) n_xmax
    write(iout2,95) w(n_xmax)
    write(iout2,75)
!
! Summarize results in main output file
!
    call ordre(nev, w1, nfin, w2)
!
    write(iuo1,5)
    write(iuo1,10)
    write(iuo1,10)
    write(iuo1,15) w2(1)
    write(iuo1,20) w2(nfin)
    write(iuo1,10)
    write(iuo1,10)
!
    if (n_eig >= 1) then
        if (n_eig == 1) then
            write(iuo1,25) ndim
        else
            write(iuo1,30) n_eig, ndim
        end if
    end if
!
    write(iuo1,65) n_xmax
    write(iuo1,70) w(n_xmax)
    write(iuo1,10)
    write(iout3,100) real(w(n_xmax)), aimag(w(n_xmax))
!
    npr = nprint/5
    write(iuo1,10)
    write(iuo1,10)
    write(iuo1,35) 5*npr
    write(iuo1,10)
    do jp = 0, npr-1
        j = 5*jp
        write(iuo1,40) w2(j+1), w2(j+2), w2(j+3), w2(j+4), w2(j+5)
    enddo
    write(iuo1,10)
    write(iuo1,10)
    write(iuo1,45) w2(1)
    write(iuo2,*) e_kin, w2(1)
    if (n_eig == 0) then
        write(iuo1,50)
    else
        write(iuo1,55)
    end if
    write(iuo1,10)
    write(iuo1,10)
    write(iuo1,60)
!
    deallocate(w, w1, w2)
!
    return
!
  5 format(/,11X,'-----------------  EIGENVALUE ANALYSIS ','-----------------')
 10 format(11X,'-',54X,'-')
 15 format(11X,'-',14X,'MAXIMUM MODULUS : ',F9.6,13X,'-')
 20 format(11X,'-',14X,'MINIMUM MODULUS : ',F9.6,13X,'-')
 25 format(11X,'-',6X,'1 EIGENVALUE IS > 1 OF A TOTAL OF ',I8,6X,'-')
 30 format(11X,'-',4X,I5,' EIGENVALUES ARE > 1 OF A TOTAL OF ',I8,2X,'-')
 35 format(11X,'-',11X,'THE ',I3,' LARGEST EIGENVALUES ARE :',11X,'-')
 40 format(11X,'-',6X,F7.4,2X,F7.4,2X,F7.4,2X,F7.4,2X,F7.4,5X,'-')
 45 format(11X,'-',4X,'SPECTRAL RADIUS OF THE KERNEL MATRIX : ',F8.5,3X,'-')
 50 format(11X,'-',5X,'---> THE MULTIPLE SCATTERING SERIES ','CONVERGES',4X,'-')
 55 format(11X,'-',10X,'---> NO CONVERGENCE OF THE MULTIPLE',9X,'-',/,11X,'-',   &
        18X,'SCATTERING SERIES',19X,'-')
 60 format(11X,'----------------------------------------','----------------',/)
 65 format(11X,'-',5X,'    LABEL OF LARGEST EIGENVALUE   : ',I5,8X,'-')
 70 format(11X,'-',5X,'    LARGEST EIGENVALUE  : ','(',F6.3,',',F6.3,')',8X,'-')
 75 format('              ')
 80 format('    KINETIC ENERGY : ',F7.2,' eV')
 85 format('    LARGEST MODULUS OF EIGENVALUE : ',F6.3)
 90 format('    LABEL OF LARGEST EIGENVALUE   : ',I5)
 95 format('    LARGEST EIGENVALUE            :  (',F6.3,',',F6.3,')')
100 format(5X,F9.5,2X,F9.5,2X,F9.5)
105 format(7X,'EIGENVALUES :',3X,'MODULUS :')
110 format(2X,'-------------------------------')
!==============================================================================!
 end subroutine eig_mat_ar
!==============================================================================!
